export interface ResolveOptions {
    /**
     * For each variable, docbits will look here for file names that match the
     * variable name (without the extension).
     * @default "_bits"
     */
    bitsDirName?: string;
    /**
     * Starting here, docbits will look in files for variables in the format
     * `${variable-name}`.
     * @default "./docs"
     */
    root?: string;
}
export interface WriteOptions {
    /**
     * The resulting files will be written here, preserving the existing folder
     * structure.
     * @default "./dist/docs"
     */
    outputDir?: string;
}
export declare enum DefaultOptions {
    BitsDirName = "_bits",
    Root = "./docs",
    OutputDir = "./dist/docs"
}
/**
 * Replaces variables in files that live within `${options.root}`.
 * Searches for variables that match the format `${bit-name}` and resolves
 * them with content from matching bits in `${options.bitsDirName}/${bit-name}*`
 * within the same folder.
 * @param options
 */
export declare function resolveBits({ bitsDirName, root, }?: ResolveOptions): AsyncGenerator<readonly [string, string], void, unknown>;
export declare function writeResult(files: ReturnType<typeof resolveBits>, { outputDir }?: WriteOptions): Promise<void[]>;
