"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeResult = exports.resolveBits = exports.DefaultOptions = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const util_1 = require("util");
const chalk_1 = require("chalk");
const globby_1 = __importDefault(require("globby"));
const mkdirp_1 = __importDefault(require("mkdirp"));
const readFile = util_1.promisify(fs_1.readFile);
const writeFile = util_1.promisify(fs_1.writeFile);
var DefaultOptions;
(function (DefaultOptions) {
    DefaultOptions["BitsDirName"] = "_bits";
    DefaultOptions["Root"] = "./docs";
    DefaultOptions["OutputDir"] = "./dist/docs";
})(DefaultOptions = exports.DefaultOptions || (exports.DefaultOptions = {}));
const bitPattern = /\${([\w\d-_]+)}/g;
/* istanbul ignore next */
/**
 * Replaces variables in files that live within `${options.root}`.
 * Searches for variables that match the format `${bit-name}` and resolves
 * them with content from matching bits in `${options.bitsDirName}/${bit-name}*`
 * within the same folder.
 * @param options
 */
async function* resolveBits({ bitsDirName = DefaultOptions.BitsDirName, root = DefaultOptions.Root, } = {}) {
    const bitPathsGlob = path_1.posix.join(root, `**/${bitsDirName}/*`);
    const bits = new Map();
    for await (const [bitPath, bitContents] of loadBits()) {
        bits.set(bitPath, bitContents);
    }
    resolveNestedBits();
    for await (const docPath of globby_1.default.stream([
        path_1.posix.join(root, '**'),
        `!${bitPathsGlob}`,
    ])) {
        const dp = docPath.toString();
        yield [
            path_1.posix.relative(root, dp),
            resolveContents((await readFile(dp)).toString(), path_1.posix.dirname(dp)),
        ];
    }
    async function* loadBits() {
        for await (const bitPath of globby_1.default.sync(bitPathsGlob)) {
            const bp = bitPath.toString();
            yield [
                path_1.posix.join(path_1.posix.dirname(bp), path_1.posix.basename(bp, path_1.posix.extname(bp))),
                (await readFile(bp)).toString(),
            ];
        }
    }
    function resolveNestedBits() {
        const bitsDirNameLength = bitsDirName.length;
        for (const [bitPath, bitContents] of bits.entries()) {
            bits.set(bitPath, resolveContents(bitContents, path_1.posix.dirname(bitPath).slice(0, bitsDirNameLength * -1)));
        }
    }
    function resolveContents(contents, docDir) {
        return contents.replace(bitPattern, (_m, bitName) => {
            const absoluteRoot = path_1.posix.resolve(root);
            let currentDir = docDir;
            while (true) {
                const bitPath = path_1.posix.join(currentDir, bitsDirName, bitName);
                const bit = bits.get(bitPath);
                if (bit) {
                    return bit;
                }
                const parentDir = path_1.posix.resolve(currentDir, '..');
                if (currentDir === absoluteRoot || currentDir === parentDir) {
                    bail();
                }
                currentDir = parentDir;
            }
            function bail() {
                throw new Error(chalk_1.red(`bit \${${bitName}} not found relative to ${path_1.posix.relative(process.cwd(), docDir)}`));
            }
        });
    }
}
exports.resolveBits = resolveBits;
async function writeResult(files, 
/* istanbul ignore next */
{ outputDir = DefaultOptions.OutputDir } = {}) {
    await mkdirp_1.default(outputDir);
    const writes = [];
    for await (const [relativePath, contents] of files) {
        const outputPath = path_1.resolve(outputDir, relativePath);
        writes.push(mkdirp_1.default(path_1.dirname(outputPath)).then(() => writeFile(outputPath, contents)));
    }
    return Promise.all(writes);
}
exports.writeResult = writeResult;
//# sourceMappingURL=resolveBits.js.map